clc; format short g; clear all; close all;

% import data from Excel
X = importdata('dataset.xlsx')


% define variables for the FAVAR
%--------------------------------------------------------------------------
t0 = 1;                                                                     % start date: 1960:Q1 
T = 235;                                                                    % end data: (196=2007:Q4) (235=2017:Q3)

% general macro variables
%------------------------
y    = log(X.data.Macro(t0:T,1))*100;                                       % y    = log of real GDP
p    = log(X.data.Macro(t0:T,2)/100)*100;                                   % p    = log of chain type GDP Price Index

i    = (log(X.data.Real(1,t0:T))*100)';                                     % i = log of real private fixed investment
q    = (log(X.data.Price(1,t0:T)/100)*100)';                                % q = log of price index for private fixed investment

ff   = X.data.Macro(t0:T,3);                                                % ff   = shadow federal funds rate (Wu-Xia)    
m1   = log(X.data.Macro(t0:T,6))*100;                                       % m1   = log of M1 money stock

ppi_fuel = log(X.data.Macro(t0:T,4)/100)*100;                               % ppi_mat = log of PPI (fuels and related products) NSA
ppi_fin  = log(X.data.Macro(t0:T,5)/100)*100;                               % ppi_fin = log of PPI (final demand for finished goods) SA

Y = [y p i q ppi_fuel-ppi_fin ff m1];
%------------------------

% disaggregate variables
%------------------------
index = [5 8 9 10 11 12 13 14 15 18 19 20 22 23 25 26 27 28 30 31 32 33 34 ...                                  % nonresidential structures
           38 39 41 42 43 44 45 47 49 50 51 52 53 54 56 59 60 61 62 65 66 68 69 71 72 73 74 76 77 78 79 ...     % nonresidential equipment
           81 82 83 ...                                                                                         % intellectual property
           87 88 90 91 92 93 ...                                                                                % residential structures
           95];                                                                                                 % residential equipment

X = [(log(X.data.Real(index,1:T))*100)' (log(X.data.Price(index,1:T))*100)']; 
%------------------------
%--------------------------------------------------------------------------


% standardize the data for principal components estimation
%--------------------------------------------------------------------------
stdY = std(Y);
stdX = std(X);

Y = (Y - repmat(mean(Y,1),T,1))./repmat(std(Y),T,1);
X = (X - repmat(mean(X,1),T,1))./repmat(std(X),T,1);
%--------------------------------------------------------------------------


% setup FAVAR and run principal components estimation
%--------------------------------------------------------------------------
N = size(X,2);                  % no. of indicators in X
M = size(Y,2);                  % no. of observed factors
K = 4;                          % no. of unobserved factors

[F,Lf,Ly] = twostep(Y,X,K);
%--------------------------------------------------------------------------


% estimate FAVAR model
%--------------------------------------------------------------------------

% estimate macro variable VAR(4) 
%-------------------------------
Spec = vgxset('n',size(Y,2),'nAR',4,'Constant',true)
n = Spec.n; plag = Spec.nAR;

[EstSpec,EstStdErrors,LLF,W] = vgxvarx(Spec,Y(plag+1:end,:),[],Y(1:plag,:));        % Warning! By default vgxvarx.m pads presample observations with zeros.      
EstSpec
%-------------------------------

% estimate factor component
%-------------------------------
Y1 = lag(Y,1); Y2 = lag(Y,2); Y3 = lag(Y,3); Y4 = lag(Y,4);
F1 = lag(F,1); F2 = lag(F,2); F3 = lag(F,3); F4 = lag(F,4);

YF = [Y1 F1 Y2 F2 Y3 F3 Y4 F4];
YF = YF(plag+1:end,:);

const     = zeros(K,1);
beta      = zeros(K,size(YF,2));
residuals = zeros(T-plag,K);
for g = 1:1:K;
    stats = regstats(F(plag+1:end,g),YF);
    const(g)       = stats.beta(1); 
    beta(g,:)      = stats.beta(2:end)';
    residuals(:,g) = stats.r;
end

% construct augmented VAR with block exogeneity  
%----------------------------------------------
for k = 1:1:plag;
    AR{k} = [EstSpec.AR{k} zeros(n,K); beta(:,(k-1)*(n+K)+1:k*(n+K))];
end;
a = [EstSpec.a; const];  

resid = [W residuals];
V = cov(resid,1);

varmodel = vgxset('a',a,'AR',AR,'Q',V)

[A0,D] = triang_fact(V);

for k = 1:1:plag;
   A{k} = A0\AR{k}; 
end;
c = A0\a; 

spec.A0 = inv(A0); spec.A  = A; spec.c  = c; spec.D  = D;
%----------------------------------------------

%--------------------------------------------------------------------------


% apply recursive identification of monetary policy shocks
%--------------------------------------------------------------------------
DynRespFunc = impulse(varmodel,spec,6,20)
%--------------------------------------------------------------------------


% obtain standardized impulse responses for [Y' F']' and X
%--------------------------------------------------------------------------
periods = 0:1:DynRespFunc.horizon;

irf_y     = -DynRespFunc.Phi{1};   
irf_p     = -DynRespFunc.Phi{2};   
irf_i     = -DynRespFunc.Phi{3};   
irf_q     = -DynRespFunc.Phi{4};   
irf_pf    = -DynRespFunc.Phi{5};   
irf_ff    = -DynRespFunc.Phi{6};   
irf_m1    = -DynRespFunc.Phi{7};   

irf_F = zeros(K,length(periods));
for h = 1:1:K;
    irf_F(h,:) = -DynRespFunc.Phi{M+h};
end;

irf_Y     = [irf_y; irf_p; irf_i; irf_q; irf_pf; irf_ff; irf_m1];
irf_X     = Lf*irf_F + Ly*irf_Y;
%--------------------------------------------------------------------------


% transform responses back into log levels of unstandardized variables
%--------------------------------------------------------------------------
for j = 1:1:length(periods);
    irf_y(j)     = stdY(1)*irf_y(j);                                        %irf_y(j)     = exp(irf_y(j)) - 1;
    irf_p(j)     = stdY(2)*irf_p(j);                                        %irf_p(j)     = exp(irf_p(j)) - 1;
    irf_i(j)     = stdY(3)*irf_i(j);                                        %irf_i(j)     = exp(irf_i(j)) - 1;
    irf_q(j)     = stdY(4)*irf_q(j);                                        %irf_q(j)     = exp(irf_q(j)) - 1;
    irf_pf(j)    = stdY(5)*irf_pf(j);                                       %irf_pf(j)    = exp(irf_pf(j)) - 1;
    irf_ff(j)    = stdY(6)*irf_ff(j);
    irf_m1(j)    = stdY(7)*irf_m1(j);                                       %irf_m1(j)    = exp(irf_m1(j)) - 1;
    irf_X(:,j)   = stdX'.*irf_X(:,j);                                       %irf_X(:,j)   = exp(irf_X(:,j)) - 1;
end;
%--------------------------------------------------------------------------


IImean_3 = mean(irf_X(1:(N/2),:));     save IImean_3 IImean_3
QQmean_3 = mean(irf_X((N/2)+1:end,:)); save QQmean_3 QQmean_3
IIstdv_3 = std(irf_X(1:(N/2),:));      save IIstdv_3 IIstdv_3
QQstdv_3 = std(irf_X((N/2)+1:end,:));  save QQstdv_3 QQstdv_3








