function errorbands = irfbands_guassian(DataMatrix,EstSpec,DynRespFunc,alpha,trials,burn,discard);
%--------------------------------------------------------------------------
% PURPOSE: computes confidence bands for orthogonalized impulse reponse 
%          functions from a VAR using the empirical distribution implied by 
%          monte carlo draws from the asymptotic guassian distribution of 
%          the VAR coefficient estimates and the residual covariance matrix
%--------------------------------------------------------------------------
% USAGE:   errorbands  = irfbands_guassian(DataMatrix,EstSpec,DynRespFunc,alpha,trials,burn,discard) 
% where:   DataMatrix  = a (T+p) x n matrix of observable data
%                        T - sample size
%                        p - presample observations (no. of AR lags)
%                        n - no. of variables in y(t)
%          EstSpec     = a vgxvarx.m output 'structure'
%          DynRespFunc = an impulse.m output 'structure'
%          alpha       = significance level (e.g., alpha = 0.10 => 5%-95%)
%          trials      = no. of monte carlo trials
%          burn        = no. of discarded initial simulations
%          discard     = {0,1} (0 = do not discard unstable draws, 1 = discard all unstable draws)
%--------------------------------------------------------------------------
% RETURNS a 'structure'
% errorbands.LB        = (alpha/2)% lower bound of the empirical distribution 
%                        of the impulse response function for each variable
% errorbands.UB        = (1-(alpha/2))% upper bound of the empirical distribution
%                        of the impulse response function for each variable
% errorbands.accept    = {"trials" if "discard"=0, no. of monte carlo draws
%                         that result in a stable reduced-form VAR if
%                         "discard"=1}
% errorbands.reject    = no. of monte carlo draws that result in an
%                        unstable reduced-form VAR
%--------------------------------------------------------------------------
% written by: Gregory Givens
%             1 March 2013
%--------------------------------------------------------------------------




% Find asymptotic covariance matrix of VAR coefficient estimates
%--------------------------------------------------------------------------
s = size(DataMatrix,1);        % sample size
p = EstSpec.nAR;               % no. of pre-sample periods
T = s - p;                     % no. of usable periods
n = EstSpec.n;                 % no. of variables in VAR 

Y = DataMatrix';
x = zeros(n*p+1,T);
for i = 1:1:T;
    temp1 = Y(:,i:i+p-1);
    temp2 = temp1(:,end:-1:1);
    temp3 = vec(temp2);
    x(:,i) = [1; temp3];
end;

SSX = zeros(n*p+1,n*p+1);
for j = 1:1:T;
    SSX = x(:,j)*x(:,j)' + SSX;
end;

Covx = kron(EstSpec.Q,inv(SSX));
%--------------------------------------------------------------------------


% Find asymptotic covariance matrix of residual-error covariance matrix
%--------------------------------------------------------------------------
Dn = duplication(n); Dn_p = (Dn'*Dn)\Dn';

Sigma22 = (1/T)*2*Dn_p*kron(EstSpec.Q,EstSpec.Q)*Dn_p';
%--------------------------------------------------------------------------


% Perform Monte Carlo simulations to obtain confidence bands
%--------------------------------------------------------------------------
k = DynRespFunc.shock;                      % kth element of the structural error vector
T = DynRespFunc.horizon;                    % response function horizon

c = EstSpec.a;                              % intercepts
for i = 1:1:p;
    AR(:,:,i) = EstSpec.AR{i};
end;
A = [c reshape(AR,n,n*p)];                  % VAR coefficients
Omega = EstSpec.Q;                          % residual covariance matrix

mu  = [vec(A'); vech(Omega)];
Cov = [Covx zeros((n*p+1)*n,n*(n+1)/2); zeros(n*(n+1)/2,(n*p+1)*n) Sigma22]; 
Psi = multirandn(mu,Cov,burn+trials);

locate = zeros(trials,1);
for j = 1:1:trials;
    
    Atemp = Psi(1:(n*p+1)*n,burn+j);
    Atemp = reshape(Atemp,n*p+1,n); AAtemp = Atemp(2:end,:);
    
    Omegatemp = Dn*Psi((n*p+1)*n+1:(n*p+1)*n+n*(n+1)/2,burn+j);
    Omegatemp = reshape(Omegatemp,n,n);
        
    ctemp = Atemp(1,:)';
    Qtemp = Omegatemp;
    for h = 1:1:p;
        ARtemp{h} = AAtemp((h-1)*n+1:(h-1)*n+n,:)';
    end;
    armodel  = vgxset('a',ctemp,'AR',ARtemp,'Q',Qtemp);  % construct a new VAR(p) for each draw      
    results1 = recursiveness(armodel);                   % perform (lower triangular) identfication for each draw
    results2 = impulse(armodel,results1,k,T);            % compute orthogonalized dynamic response functions for each draw
    
    % track unstable draws
    %---------------------
    companion = [AAtemp'; eye(n*(p-1)) zeros(n*(p-1),n)];
    E = abs(eig(companion));
    if sum(E>=1.00) ~= 0;
        locate(j) = 1;   
    end;
    bad = sum(locate);
    %---------------------
    
    for g = 1:1:n;
        RespFuncs(g,:,j) = results2.Phi{g};
    end;
    
end;

if discard == 0;
    RespFuncs = RespFuncs;
else
    RespFuncs = RespFuncs(:,:,find(locate==0));
end;

accept = size(RespFuncs,3);    
RespFuncs1 = sort(RespFuncs,3,'ascend');                % sort response functions 'element-by-element' in ascending order
%--------------------------------------------------------------------------
   

% Stack results in an 'output' structure
%--------------------------------------------------------------------------
lower = ceil((alpha/2)*accept);
upper = floor((1-(alpha/2))*accept);

for i = 1:1:n;
    LB{i} = RespFuncs1(i,:,lower+1);
    UB{i} = RespFuncs1(i,:,upper);
end;

errorbands.Resp   = RespFuncs;
errorbands.LB     = LB;
errorbands.UB     = UB;
errorbands.accept = accept;
errorbands.reject = bad;
%--------------------------------------------------------------------------



