function Decomp = vardecomp(EstSpec,StructSpec,k,s);
%--------------------------------------------------------------------------
% PURPOSE: decomposes the s-step-ahead forecast error variance of the 
%          orthogonalized VAR innovations         
%--------------------------------------------------------------------------
% USAGE:   Decomp      = vardecomp(EstSpec,StructSpec,k,s) 
% where:   EstSpec     = a vgxvarx.m output 'structure'
%          StructSpec  = a recursiveness.m output 'structure'
%          k           = kth orthogonalized VAR innovation
%          s           = forecast horizon 
%--------------------------------------------------------------------------
% RETURNS a 'structure'
% Decomp.fevd          = (nx1) vector containing the contribution of the 
%                        kth orthogonalized innovation to the s-step-ahead 
%                        forecast error of each of the n variables 
%                        contained in the VAR 
% Decom.fevdLR         = (nx1) vector containing the contribution of the 
%                        kth orthogonalized innovation to the unconditional
%                        variance of each of the n variables contained in 
%                        the VAR
% Decomp.conv          = convergence criteria for sylvester equations
% Decomp.shock         = k
% Decomp.horizon       = s (as s -> infinity), MSE of the s-step-ahead 
%                        forecast approaches the unconditional variance 
%--------------------------------------------------------------------------
% written by: Gregory Givens
%             15 May 2013
%--------------------------------------------------------------------------



Decomp.method = 'contribution of the kth orthogonalized innovation to the s-step-ahead VAR forecast error';

n = EstSpec.n;      q = EstSpec.nAR;
A0 = StructSpec.A0; D = StructSpec.D;

for i = 1:1:q;
   A(:,:,i) = EstSpec.AR{i};
end;
A = reshape(A,n,n*q);
B = [A; eye(n*(q-1)) zeros(n*(q-1),n)];     % builds companion matrix

a = inv(A0); 
select = [eye(n) zeros(n,n*(q-1))]; 

% conditional forecast error decomposition
Fevd = zeros(n,n,n);
for i = 1:1:n;
    fevd = zeros(n,n);
    for j = 1:1:s;                
        Psij = select*B^(j-1)*select'; 
        fevd = fevd + D(i,i)*Psij*a(:,i)*a(:,i)'*Psij';
    end;
    Fevd(:,:,i) = fevd;
end;

MSE = sum(Fevd,3);
mse = 100*Fevd(:,:,k)./MSE;

% unconditional variance decomposition
vecS_y  = (eye((n*q)^2) - kron(B,B))\vec([EstSpec.Q zeros(n,n*(q-1)); zeros(n*(q-1),n*q)]); 
S_y     = reshape(vecS_y,n*q,n*q); 
Sigma_y = select*S_y*select';
[S_y,dd1]     = sylvester([EstSpec.Q zeros(n,n*(q-1)); zeros(n*(q-1),n*q)],B,1);

Q      = [a(:,k); zeros(n*(q-1),1)]*[a(:,k); zeros(n*(q-1),1)]'*D(k,k);
vecS   = (eye((n*q)^2) - kron(B,B))\vec(Q); 
S      = reshape(vecS,n*q,n*q); 
Sigma  = select*S*select';
[S,dd2]       = sylvester(Q,B,1);

Decomp.fevd    = diag(mse);
Decomp.fevdLR  = diag(100*Sigma./Sigma_y);
Decomp.conv    = [dd1 dd2];
Decomp.shock   = k;
Decomp.horizon = s;


